#[cfg(feature = "server")]
use crate::util::surrealdb::{thing_to_string, DB};
use serde::{Deserialize, Serialize};

#[cfg(feature = "server")]
mod migration;

#[derive(Debug, Deserialize, Serialize, PartialEq, Eq, Clone)]
pub struct Member {
    #[cfg_attr(feature = "server", serde(deserialize_with = "thing_to_string"))]
    pub id: String,
    pub name: Name,
    pub hours: Vec<String>,
    pub groups: Vec<String>,
    pub diploma: Option<String>,
    pub registration_token: Option<String>,
}

#[derive(Debug, Deserialize, Serialize, PartialEq, Eq, Clone)]
pub struct Name {
    pub first: String,
    pub full: String,
}

#[derive(Debug, Deserialize, Serialize, PartialEq, Clone)]
pub struct MembersMigration {
    pub inserted: Vec<Member>,
    pub updated: Vec<Member>,
    pub removed: Vec<Member>,
}

#[cfg(feature = "server")]
impl Member {
    pub async fn fetch_all() -> Result<Vec<Self>, surrealdb::Error> {
        let mut res = DB.query("SELECT id, name.first, name.full, hours, groups, diploma, registration_token FROM member").await?;

        let members: Vec<Self> = res.take(0)?;

        Ok(members)
    }

    pub async fn fetch_from_registration_token(
        registration_token: String,
    ) -> Result<Option<Self>, surrealdb::Error> {
        let mut res = DB
            .query("SELECT id, name.first, name.full, hours, groups, diploma, registration_token FROM ONLY member WHERE registration_token = $registration_token LIMIT 1")
            .bind(("registration_token", registration_token))
            .await?;

        let member: Option<Self> = res.take(0)?;

        Ok(member)
    }

    pub async fn fetch_from_user(user_id: String) -> Result<Vec<Self>, crate::Error> {
        let mut res = DB
            .query(
                "SELECT VALUE ->user_to_member->member.* FROM ONLY type::thing('user', $user_id)",
            )
            .bind(("user_id", user_id))
            .await?;

        tracing::info!("{res:?}");

        let members: Vec<Self> = res.take(0)?;

        Ok(members)
    }
}

impl MembersMigration {
    pub fn new() -> MembersMigration {
        MembersMigration {
            inserted: vec![],
            updated: vec![],
            removed: vec![],
        }
    }
}
